@@ -1,5 +1,5 @@
 d = dict
-from hypergen.context import context as c
+from hypergen.context import context as c, contextlist
 from hypergen.hypergen import *
 from hypergen.hypergen import make_string, t
 from hypergen.plugins.appstate import AppstatePlugin
@@ -39,8 +39,8 @@ __all__ = [
     "optgroup", "option", "output", "p", "param", "picture", "pre", "progress", "q", "rp", "rt", "ruby", "s", "samp",
     "script", "section", "select", "small", "source", "span", "strike", "strong", "style", "sub", "summary", "sup",
     "svg", "table", "tbody", "td", "template", "textarea", "tfoot", "th", "thead", "time", "title", "tr", "track",
-    "tt", "u", "ul", "var", "video", "wbr", "component", "hypergen", "raw", "write", "rst", "TemplatePlugin", "HTML",
-    "FULL", "COMMANDS", "base_element", "OMIT"]
+    "tt", "u", "ul", "var", "video", "wbr", "component", "hypergen", "raw", "write", "rst", "HTML", "FULL",
+    "COMMANDS", "OMIT", "hypergen_to_response"]
 
 ### Constants ###
 
@@ -51,7 +51,7 @@ OMIT = "__OMIT__"
 class TemplatePlugin:
     @contextmanager
     def context(self):
-        with c(at="hypergen", into=[], ids=set()):
+        with c(at="hypergen", into=contextlist("target_id"), ids=set()):
             yield
 
 ### Rendering ###
@@ -109,6 +109,9 @@ def hypergen(func, *args, **kwargs):
             elif returns == FULL:
                 return d(html=html, context=c.clone(), func_result=func_result)
 
+def hypergen_to_response(func, *args, **kwargs):
+    return HttpResponse(hypergen(func, *args, **kwargs))
+
 ### Helpers ###
 def join_html(html):
     def fmt(html):
@@ -161,6 +164,8 @@ class base_element(ContextDecorator):
             self.t = attrs.pop("t", t)
             self.children = children
             self.attrs = attrs
+            self.sep = attrs.pop("sep", "")
+            self.end_char = attrs.pop("end", None)
 
             # Id
             id_ = self.attrs.get("id_", self.attrs.pop("id", None))
@@ -169,15 +174,12 @@ class base_element(ContextDecorator):
             self.attrs["id_"] = id_
 
             # Make sure ids are unique
-            self.i = len(c.hypergen.into)
-            if self.attrs["id_"] is not None:
-                id_ = self.attrs["id_"]
+            if id_ is not None:
                 assert id_ not in c.hypergen["ids"], "Duplicate id: {}".format(id_)
                 c.hypergen["ids"].add(id_)
 
-            self.sep = attrs.pop("sep", "")
-            self.end_char = attrs.pop("end", None)
-
+            # Render and save position in into.
+            self.i = len(c.hypergen.into)
             c.hypergen.into.extend(self.start())
             c.hypergen.into.extend(self.end())
             self.j = len(c.hypergen.into)
@@ -220,7 +222,8 @@ class base_element(ContextDecorator):
 
         def signature(a, kw):
             a, kw = deepcopy(a), deepcopy(kw)
-            return ", ".join([args(x) for x in a] + ["{}={}".format(*kwargs(k, v)) for k, v in list(kw.items())])
+            return ", ".join([args(x)
+                for x in a] + ["{}={}".format(*kwargs(k, v)) for k, v in list(kw.items()) if v is not None])
 
         return "{}({})".format(self.__class__.__name__, signature(self.children, self.attrs))
 
@@ -290,6 +293,10 @@ class base_element(ContextDecorator):
             return [" ", k, '="', v, '"']
 
     def start(self):
+        cache = getattr(self, "_start_cache", None)
+        if cache:
+            return cache
+
         into = ["<", self.tag]
         for k, v in self.attrs.items():
             into.extend(self.attribute(k, v))
@@ -299,6 +306,8 @@ class base_element(ContextDecorator):
         into.append('>')
         into.extend(self.format_children(self.children))
 
+        self._start_cache = into
+
         return into
 
     def end(self):
@@ -323,7 +332,7 @@ class Component(object):
 def component(f):
     @wraps(f)
     def _(*args, **kwargs):
-        with c(into=[], at="hypergen"):
+        with c(into=contextlist("target_id"), at="hypergen"):
             f(*args, **kwargs)
             into = c.hypergen.into
         i = len(c.hypergen.into)
@@ -340,7 +349,8 @@ class input_(base_element_void):
 
     def __init__(self, *children, **attrs):
         type_ = attrs.get("type_", attrs.pop("type", None))
-        attrs["type_"] = type_
+        if type_:
+            attrs["type_"] = type_
         if type_ == "radio":
             assert attrs.get("name"), "Name must be set for radio buttons."
         super(input_, self).__init__(*children, **attrs)
